/*
 * cpudists - print CPU time distributions by Kernel/Idle/Processes.
 *            Written using DTrace (Solaris 10 3/05).
 *
 * FIELDS: 
 *	value           The following or the process name,
 *	IDLE            Idle time - CPU running idle thread
 *	KERNEL          Kernel time - Kernel servicing interrupts, ...
 *	PROCESS         Process time - PIDs running on the system
 *	count           Number of occurances at least this duration (ns)
 *
 * NOTES:
 * * This takes into account multiple CPU servers, the total 
 * seconds consumed will be a multiple of the CPU count and interval.
 *
 * SEE ALSO: cputimes
 *
 * COPYRIGHT: Copyright (c) 2005 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Author: Brendan Gregg  [Sydney, Australia]
 *
 * 27-Apr-2005   Brendan Gregg   Created this.
 * 22-Sep-2005	   "      "	Fixed key corruption bug.
 * 22-Sep-2005	   "      "	Last update.
 * 23-Oct-2008   Tom Erickson   (adapted to Chime)
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/* Initialise variables */
dtrace:::BEGIN
{
	cpustart[cpu] = 0;
}

/* Flag this thread as idle */
sysinfo:unix:idle_enter:idlethread
{
	idle[cpu] = 1;
}

/* Save kernel time between running threads */
sched:::on-cpu 
/cpustart[cpu]/
{
	this->elapsed = timestamp - cpustart[cpu];
	@Procs["KERNEL"] = quantize(this->elapsed);
}

/* Save the elapsed time of a thread */
sched:::off-cpu,
sched:::remain-cpu,
profile:::profile-1sec
/cpustart[cpu]/
{
	/* determine the name for this thread */
	program[cpu] = ((pid == 0)
	    ? (idle[cpu] ? "IDLE" : "KERNEL")
	    : ??PROCESS??);

	/* save elapsed */
	this->elapsed = timestamp - cpustart[cpu];
	@Procs[program[cpu]] = quantize(this->elapsed);
	cpustart[cpu] = timestamp;
}

/* Record the start time of a thread */
sched:::on-cpu,
sched:::remain-cpu
{
	idle[cpu] = 0;
	cpustart[cpu] = timestamp;
}

??PROCESS "PROCESS"
